@@ -6,22 +6,25 @@ loss.function = function(fitted, data, loss, extra.args, debug = FALSE) {
     result = entropy.loss(fitted = fitted, data = data, debug = debug)
 
   }#THEN
-  else if (loss == "pred") {
+  else if (loss %in% c("pred", "pred-lw")) {
 
     result = classification.error(node = extra.args$target, fitted = fitted,
-               prior = extra.args$prior, data = data, debug = debug)
+               prior = extra.args$prior, n = extra.args$n,
+               from = extra.args$from, data = data, loss = loss, debug = debug)
 
   }#THEN
-  else if (loss == "cor") {
+  else if (loss %in% c("cor", "cor-lw")) {
 
     result = predictive.correlation(node = extra.args$target, fitted = fitted,
-               data = data, debug = debug)
+               n = extra.args$n, from = extra.args$from, data = data,
+               loss = loss, debug = debug)
 
   }#THEN
-  else if (loss == "mse") {
+  else if (loss %in% c("mse", "mse-lw")) {
 
     result = mean.square.error(node = extra.args$target, fitted = fitted,
-               data = data, debug = debug)
+               n = extra.args$n, from = extra.args$from, data = data,
+               loss = loss, debug = debug)
 
   }#THEN
 
@@ -35,7 +38,7 @@ loss.function = function(fitted, data, loss, extra.args, debug = FALSE) {
 # how to aggregate loss functions across the folds of cross-validation.
 kfold.loss.postprocess = function(kcv, kcv.length, loss, extra.args, data) {
 
-  if (loss == "cor") {
+  if (loss %in% c("cor", "cor-lw")) {
 
     if (all(is.na(unlist(sapply(kcv, "[", "loss"))))) {
 
@@ -68,9 +71,12 @@ kfold.loss.postprocess = function(kcv, kcv.length, loss, extra.args, data) {
 }#LOSS.POSTPROCESS
 
 # predictive mean square error for gaussian networks.
-mean.square.error = function(node, fitted, data, debug = FALSE) {
+mean.square.error = function(node, fitted, n, from, data, loss, debug = FALSE) {
 
-  pred = gaussian.prediction(node, fitted, data)
+  if (loss == "mse")
+    pred = gaussian.prediction(node, fitted, data)
+  else if (loss == "mse-lw")
+    pred = map.prediction(node, fitted, data, n = n, from = from)
 
   return(list(loss = mean((data[, node] - pred)^2), predicted = pred,
     observed = data[, node]))
@@ -78,11 +84,15 @@ mean.square.error = function(node, fitted, data, debug = FALSE) {
 }#MEAN.SQUARE.ERROR
 
 # predictive correlation for gaussian networks.
-predictive.correlation = function(node, fitted, data, debug = FALSE) {
+predictive.correlation = function(node, fitted, n, from, data, loss,
+    debug = FALSE) {
 
-  pred = gaussian.prediction(node, fitted, data)
+  if (loss == "cor")
+    pred = gaussian.prediction(node, fitted, data)
+  else if (loss == "cor-lw")
+    pred = map.prediction(node, fitted, data, n = n, from = from)
 
-  if (length(fitted[[node]]$parents) == 0)
+  if (((loss == "cor") && length(fitted[[node]]$parents) == 0))
     return(list(loss = NA, predictions = pred, observed = data[, node]))
   else
     return(list(loss = cor(data[, node], pred), predicted = pred,
@@ -104,12 +114,15 @@ entropy.loss = function(fitted, data, keep = names(fitted), by.sample = FALSE,
 }#ENTROPY.LOSS
 
 # classification error as a loss function.
-classification.error = function(node, fitted, prior = NULL, data, debug = FALSE) {
+classification.error = function(node, fitted, prior = NULL, n, from, data,
+    loss, debug = FALSE) {
 
   if (is(fitted, c("bn.naive", "bn.tan")))
     pred = naive.classifier(node, fitted, prior, data)
-  else
+  else if (loss == "pred")
     pred = discrete.prediction(node, fitted, data)
+  else if (loss == "pred-lw")
+    pred = map.prediction(node, fitted, data, n = n, from = from)
 
   l = .Call("class_err",
             reference = minimal.data.frame.column(data, node),
